import java.awt.Point;

/**
 * This class is the basic layer for any moving object in Xonix.
 * 
 * @see Ball
 * @see Spider
 */
class Movable {
	/**
	 * Center of the object - current position.
	 */
	protected Point center;
	/**
	 * Last position of the center.
	 */
	protected Point last;
	/**
	 * Horizontal velocity
	 */
	protected int dx;
	/**
	 * Vertical velocity
	 */
	protected int dy;
	/**
	 * Dirty - needs repainting.
	 */
	protected boolean dirty;
	/**
	 * Level map.
	 */
	protected Map map;
	/**
	 * Radius of the object. All objects in the game have equal size.
	 */
	public static int radius = 8;

	/**
	 * Makes a moving object.
	 * 
	 * @param startx
	 *            Initial object center x coordinate.
	 * @param starty
	 *            Initial object center y coordinate.
	 * @param speedx
	 *            Initial horizontal velocity.
	 * @param speedy
	 *            Initial vertical velocity.
	 * @param _map
	 *            Reference to game map.
	 */
	public Movable(int startx, int starty, int speedx, int speedy, Map _map) {
		center = new Point(Map.round(startx), Map.round(starty));
		last = new Point(-1, -1);
		dx = Map.round(speedx);
		dy = Map.round(speedy);
		dirty = true;
		map = _map;
	}

	/**
	 * Moves the object according its speed.
	 */
	public void move() {
		center.translate(dx, dy);
	}

	/**
	 * Moves the object back with current speed.
	 */
	public void unmove() {
		center.translate(-dx, -dy);
	}

	/**
	 * Remember current position of the object.
	 */
	public void remember_last() {
		last.move(center.x, center.y);
	}

	public int getx() {
		return center.x;
	}

	public int gety() {
		return center.y;
	}

	public int getdx() {
		return dx;
	}

	public int getdy() {
		return dy;
	}

	public Point get_center() {
		return center;
	}

	/**
	 * Checks if the point is inside the object.
	 * 
	 * @param p
	 *            The point to check.
	 * @return <strong>true</strong> if the point is inside object,<br>
	 *         <strong>false</strong> otherwise.
	 */
	public boolean collides(Point p) {
		if (Math.abs(getx() - p.x) <= radius
				&& Math.abs(gety() - p.y) <= radius)
			return true;
		return false;
	}

	/**
	 * Checks if two objects collide.
	 * 
	 * @param other
	 *            Object to check.
	 * @return <strong>true</strong> if the objects collide,<br>
	 *         <strong>false</strong> otherwise.
	 */
	public boolean collides(Movable other) {
		if (Math.abs(getx() - other.getx()) <= 2 * radius
				&& Math.abs(gety() - other.gety()) <= 2 * radius)
			return true;
		return false;
	}

	/**
	 * Moves the object to a new location.
	 * 
	 * @param x
	 *            New x coordinate.
	 * @param y
	 *            New y coordinate.
	 */
	public void move(int x, int y) {
		center.move(x, y);
	}

}
