import java.awt.Graphics;
import java.awt.Point;
import java.util.Vector;

/**
 * This class implements <font color="green">The Brave Spider</font>.<br>
 * Here is his portrait: <img src="gif/spider.gif" alt="portrait of the WBC"
 * height="16" width="16">
 */
public class Spider extends Movable {
	private int current_wall;
	private boolean isOut;
	private boolean isOutside;
	private final static int FROZEN = 0;
	private final static int UP = 1;
	private final static int DOWN = 2;
	private final static int LEFT = 3;
	private final static int RIGHT = 4;
	private int direction;
	private boolean has_path;
	private Vector path;
	private boolean can_out;

	/**
	 * Creates a new spider.
	 * 
	 * @param startx
	 *            Initial x coordinate.
	 * @param starty
	 *            Initial y coordinate.
	 * @param _map
	 *            Reference to the level map.
	 */
	public Spider(int startx, int starty, Map _map) {
		super(startx, starty, 0, 0, _map);
		set_current_wall();
		setDirection(FROZEN);
		path = new Vector();
		has_path = false;
		can_out = true;
	}

	private void set_current_wall() {
		current_wall = -1;
		isOut = false;
		isOutside = false;
		isOut = (map.position(get_center()) != Map.COLLISION);
		isOutside = (map.position(get_center()) == Map.OUTSIDE);
	}

	private void update_path() {
		map.set(get_center(), Map.PATH);
		if (path.size() == 0 && isOut) {
			int dir = direction;
			setDirection(FROZEN);
			setDirection(dir);
		}
		switch (direction) {
		case UP:
			((Wall) path.lastElement()).set_from(center.y);
			break;
		case DOWN:
			((Wall) path.lastElement()).set_to(center.y);
			break;
		case LEFT:
			((Wall) path.lastElement()).set_from(center.x);
			break;
		case RIGHT:
			((Wall) path.lastElement()).set_to(center.x);
			break;
		}
	}

	private void finish_path() {
		for (int i = 0; i < path.size(); ++i) {
			Wall wl = (Wall) (path.elementAt(i));
			if (wl.get_type() == Wall.LOWER || wl.get_type() == Wall.UPPER) {
				for (int x = wl.get_from(); x <= wl.get_to(); ++x)
					map.set(x, wl.get_base(), Map.WALL);
			}
			if (wl.get_type() == Wall.LEFT || wl.get_type() == Wall.RIGHT) {
				for (int y = wl.get_from(); y <= wl.get_to(); ++y)
					map.set(wl.get_base(), y, Map.WALL);
			}
		}

		freeze();
		setDirection(FROZEN);
		has_path = true;
		dirty = true;
	}

	/**
	 * Checks if the spider has a path completed.
	 */
	public boolean has_path() {
		return has_path;
	}

	/**
	 * Cleans the path.
	 */
	public void clear_path() {
		path = new Vector();
		has_path = false;
		System.gc();
	}

	private void setDirection(int newdir) {
		if (isOut && newdir != direction && newdir != FROZEN) {
			switch (newdir) {
			case UP:
			case DOWN:
				path.addElement(new Wall(last.y, center.y, center.x, Wall.LEFT,
						true));
				break;
			case LEFT:
			case RIGHT:
				path.addElement(new Wall(last.x, center.x, center.y,
						Wall.UPPER, true));
				break;
			}
		}
		direction = newdir;
	}

	/**
	 * Makes the spider to move up.
	 */
	public void up() {
		dy = -Map.COEFF;
		dx = 0;
		setDirection(UP);
	}

	/**
	 * Makes the spider to move down.
	 */
	public void down() {
		dy = Map.COEFF;
		dx = 0;
		setDirection(DOWN);
	}

	/**
	 * Makes the spider to move left.
	 */
	public void left() {
		dy = 0;
		dx = -Map.COEFF;
		setDirection(LEFT);
	}

	/**
	 * Makes the spider to move right.
	 */
	public void right() {
		dy = 0;
		dx = Map.COEFF;
		setDirection(RIGHT);
	}

	/**
	 * Makes the spider to stay still.
	 */
	public void freeze() {
		dx = dy = 0;
		can_out = true;
	}

	/**
	 * Undoes spider move.
	 */
	public void unmove() {
		super.unmove();
		freeze();
		set_current_wall();
	}

	/**
	 * Makes a move.
	 */
	public void move() {
		super.move();
		dirty = true;
		boolean was_out = isOut;
		set_current_wall();

		if (isOutside) {
			// go back, buddy, you should be inside the field
			unmove();
			return;
		}
		if (isOut) {
			if (!was_out && !can_out) {
				unmove();
				return;
			} else
				update_path();
		} else {
			if (was_out)
				finish_path();
		}
		if (dx != 0 || dy != 0)
			can_out = false;
	}

	/**
	 * Update spider picture.
	 */
	public void update(Graphics g, Xonix parent) {
		if (!dirty)
			return;
		map.updateRect(g, last.x - radius, last.y - radius, last.x + radius,
				last.y + radius, parent);
		if (isOut)
			g.drawImage(Xonix.spiderImages[1], center.x - radius, center.y
					- radius, radius * 2, radius * 2, parent);
		else
			g.drawImage(Xonix.spiderImages[0], center.x - radius, center.y
					- radius, radius * 2, radius * 2, parent);
		remember_last();
		dirty = false;
	}

	/**
	 * Draw the spider.
	 */
	public void paint(Graphics g, Xonix parent) {
		dirty = true;
		update(g, parent);
	}

	/**
	 * Check if the ball doesn't cross the path.
	 */
	public boolean check_collision(Ball b) {
		Point bogus = new Point(0, 0);
		if (!isOut)
			return false; // no path - nothing to check
		if (map.find(b, bogus, Map.CROSSPATH))
			return true;
		return false;
	}
}
